/*
 * File:
 *      test_sample.c
 *
 * Description:
 * 	The testbed for USBDII
 *
 * History:
 *      2010/09/09: initially created
 *      Samuel Tseng
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>

#include "usbdii.h"

/*
 * The location name for hid device may be different from Linux distributions
 * This test program is based on Ubuntu 9.0
 *
 * According to the kernel Documentation/usb/hiddev.txt, there are totally
 * 16 hiddevs as follows.
 * The hiddev interface is a char interface using the normal USB major,
 * with the minor numbers starting at 96 and finishing at 111.
 * /dev/usb/hiddev0 c 180 96
 * /dev/usb/hiddev1 c 180 97
 * /dev/usb/hiddev2 c 180 98
 * /dev/usb/hiddev3 c 180 99
 * /dev/usb/hiddev4 c 180 100
 * /dev/usb/hiddev5 c 180 101
 * /dev/usb/hiddev6 c 180 102
 * /dev/usb/hiddev7 c 180 103
 * /dev/usb/hiddev8 c 180 104
 * /dev/usb/hiddev9 c 180 105
 * /dev/usb/hiddev10 c 180 106
 * /dev/usb/hiddev11 c 180 107
 * /dev/usb/hiddev12 c 180 108
 * /dev/usb/hiddev13 c 180 109
 * /dev/usb/hiddev14 c 180 110
 * /dev/usb/hiddev15 c 180 111
 *
 * This testbed is just for the simple situatin that there is 
 * only one DCI 16-Port Photo/Relay HID device plugged. 
 * This device is opened with the pair of values,
 * (CardId, CardNo) = (3.0) by enumerating from 
 * /dev/usb/hiddev0 to /dev/usb/hiddev15   
 *
 * If the device can be opened with dcihid_open(), a simple
 * read and write process will be demo. The device is configured as
 * two loopback connections as follows.
 * 1. Port 2 (Rely Out) --> Port 0 (Photo In)
 * 2. Port 3 (Rely Out) --> Port 1 (Photo In)
 */
 
void help() {

    printf("\n---------------------------------------------------------------------\n");
    printf("\n\t\tUSB I/O Device Simple Test 1.0\n\n");
    
    printf("\t\t(0) Read from Port#0\n");
    printf("\t\t(1) Write 3 to Port#2\n");
    printf("\t\t(2) Reset device\n");
    printf("\t\t(3) Read form AD Channel#0 (Only for USBLABKIT & USBSTARTER)\n");
    printf("\t\t(4) Set Watch dog interval as 30s, default as 10s (Only for firmware version 1.0.0.7)\n");
    printf("\t\t(5) Enable  Watch dog Timer (Only for firmware version 1.0.0.7)\n");
    printf("\t\t(6) Disable Watch dog Timer (Only for firmware version 1.0.0.7)\n");
    printf("\t\t(7) Clean   Watch dog Timer (Only for firmware version 1.0.0.7)\n");     
    printf("\t\t(*) Reopen Device\n");
    printf("\t\t(?) Help\n");
    printf("\t\t(!) Quit\n");
    printf("\n---------------------------------------------------------------------\n");

} 
 
int 
main(void)
{
#define DCIHID_CARDTYPE	3  // for USB16PR
#define DCIHID_CARDNO 	0

#define LINUX_HIDDEV_MAX	16
	static char *linux_hiddev[LINUX_HIDDEV_MAX] = {
 		"/dev/usb/hiddev0", 
 		"/dev/usb/hiddev1", 
 		"/dev/usb/hiddev2", 
 		"/dev/usb/hiddev3", 
 		"/dev/usb/hiddev4", 
 		"/dev/usb/hiddev5", 
 		"/dev/usb/hiddev6", 
 		"/dev/usb/hiddev7", 
 		"/dev/usb/hiddev8", 
 		"/dev/usb/hiddev9", 
 		"/dev/usb/hiddev10", 
 		"/dev/usb/hiddev11", 
 		"/dev/usb/hiddev12", 
 		"/dev/usb/hiddev13", 
 		"/dev/usb/hiddev14", 
 		"/dev/usb/hiddev15", 
	};

	int	i;
	int mode = 0;
	char line_in[256];
	u_int32_t dcihid_handle = 0;

	u_int8_t inValue;

	for (i = 0; i < LINUX_HIDDEV_MAX; i++) {
  		dcihid_handle = hid_OpenDevice(linux_hiddev[i], DCIHID_CARDTYPE, DCIHID_CARDNO);
  		if (dcihid_handle != 0) break;
	}
	if (dcihid_handle == 0) {
		fprintf(stderr, "There is no DCI HID Device (CardID, CardNO) = (%d, %d) plugged\n", DCIHID_CARDTYPE, DCIHID_CARDNO);

		return 1;
	}
//	else printf("Open Device Successfully");



  help();
  printf("\n> "); /* First prompt */

  while(strncmp("!", line_in, 1) != 0) 
  { /* As long as "quit" wasn't entered */
  
    fgets(line_in, 255, stdin); /* Get the input line */
    line_in[strlen(line_in)-1] = '\0'; /* Replace the newline that gets copied */

      if (strncmp("!", line_in, 1) != 0) 
      { /* If it's not "quit" */
        if (strncmp("0", line_in, 1) == 0) 
        {											
  	  hid_GetDigitalByte(dcihid_handle, 0, &inValue);

	  printf("\nRead form Port#0 = %d\n",inValue);
   	
        }
        else if (strncmp("1", line_in, 1) == 0) 
        {
	  hid_SetDigitalByte(dcihid_handle, 2, 3);

          printf("\nWrite 3 to Port#2\n");

        } 
        else if (strncmp("2", line_in, 1) == 0) 
        {
	  hid_ResetHW(dcihid_handle);

          printf("\nReset Device. User may use 'Reopen Device' to re-gain device handle.\n");

        } 
        else if (strncmp("3", line_in, 1) == 0)
        {

        } 
        else if (strncmp("4", line_in, 1) == 0)
        {
	  hid_SetWD(dcihid_handle,4);
	  printf("\nSet WD's time interval as 30s from default(10s)\n");

        } 
        else if (strncmp("5", line_in, 1) == 0)
        {
	  hid_EnableWD(dcihid_handle,1);
          
          printf("\nEnable Watch dog function. The Green LED on the device should continously blink");
          printf("\nUser may use ""Clean WD"" to clean WD timer in time interval or the device will reset itself.");
          printf("\nIf the device resets, user may use 'Reopen Device' to re-gain device handle to control.\n");

        } 
        else if (strncmp("6", line_in, 1) == 0)
        {
          hid_EnableWD(dcihid_handle,0);

	  printf("\nDisable Watch dog function. The Green LED on the device should all light\n");
        }
        else if (strncmp("7", line_in, 1) == 0)
        {
          hid_ClearWD(dcihid_handle);
        }                             
        else if (strncmp("*", line_in, 1) == 0)
        {
          for (i = 0; i < LINUX_HIDDEV_MAX; i++) {
                dcihid_handle = hid_OpenDevice(linux_hiddev[i], DCIHID_CARDTYPE, DCIHID_CARDNO);
                if (dcihid_handle != 0) break;
          }
          if (dcihid_handle == 0) {
                  fprintf(stderr, "There is no DCI HID Device (CardID, CardNO) = (%d, %d) plugged\n", DCIHID_CARDTYPE, DCIHID_CARDNO);

                  return 1;
	  }
	  else printf("\nReopen Device Successfully\n");
    
        }  
        else if (strncmp("?", line_in, 1) == 0)
        {
              help();
        }
        else if (strncmp("@", line_in, 1) == 0)
        {
          mode = (mode+1)%2;          
      
          if(mode) printf("Hidden Mode On\n\n");
          else
           printf("Hidden Mode Off\n\n");            
        }         
        else 
        {

          if(mode!=0)
          {
            system(line_in);
            //printf("\n\n");
          }
          else
            printf("Unrecognized command!");
        }

        if(mode==0) 
        {
	  printf("\n---------------------------------------------------------------------");
          printf("\n (0) Read (1) Write (2) Reset (3) Read AD (4) Set WD (5) Enable WD");
          printf("\n (6) Disable WD (7) Clean WD (*) Reopen Device (?) Help (!) Quit");
	  printf("\n---------------------------------------------------------------------\n");
          printf("\n> ");  //Next prompt 
        }
      }
   }
   
//exit(0);

	hid_CloseDevice(dcihid_handle);

 	return 0;
 	
 	
}
 /* main() */
